#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// Created by Elitecamper 
// Version 1.0
// Verson  2.0 Zondor Jun 19 2014
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// A Low Pass Filter
    /// </summary>
    [Description("Low Pass Filter Histogram")]
    public class LowPassFilter_v1 : Indicator
    {
       // #region Variables
        // why would anyone care which variables were generated by the wizard?
            private int order = 3; 
            private int filterPeriod = 35; 
            private int preSmooth = 2; 
			private	DataSeries		Smoother;
			private	DataSeries		Filter;
		    private double a, b, c;
            private double pi = Math.Round(Math.PI,6);
		    private double k1,k2,k3,k4,k5,k6;
		    private SMA sma1,sma2;
		    private Color downColor    =  Color.Red;
		    private Color upColor      =  Color.Green;
		    private Color neutralColor =  Color.Yellow;
        
       // #endregion

        
        protected override void Initialize()
        {
            Add(new Plot(Color.FromKnownColor(KnownColor.Blue), PlotStyle.Line, "Plot0"));
			Plots[0].Pen.Width = 4;
            Smoother = new DataSeries(this);
            Filter = new DataSeries(this);
            Overlay				= true;
			this.Name="LowPassFilter v1";
			BarsRequired=2;
			//MaximumBarsLookBack = MaximumBarsLookBack.Infinite;
        }

        protected override void OnStartUp()
		{  try{
			sma1=SMA(Input, PreSmooth);
			sma2=SMA(Smoother, FilterPeriod);
			switch (Order)
            {  // Why would anyone want to calculate constants over and over again on every incoming tick?
			   // Why would we want to use constants with 14 digit precision to calculate an output that needs one digit precision?
                case 1:     
                    break;

                case 2:
                    a = Math.Round(Math.Exp(-Math.Sqrt(2) * pi / FilterPeriod),6);
                    b = Math.Round(2 * a * Math.Cos(Math.Sqrt(2) * pi / FilterPeriod),6);
                    k1=a*a; k2=(1 - b + a * a);
                    break;

                case 3:
                    a = Math.Round(Math.Exp(-pi / FilterPeriod),6);
                    b = Math.Round(2 * a * Math.Cos(Math.Sqrt(3) * pi / FilterPeriod),6);
                    c = Math.Round(Math.Exp(-2 * pi / FilterPeriod),6);
                    k3=(b + c); k4=Math.Round((c + b * c),6);k5=Math.Round(c * c,6);k6=Math.Round(((1 - b + c) * (1 - c)),6);
                    break;
            }
			}catch (Exception e){	Print("Low Pass Cycle line 75 bar "+CurrentBars[0]+" : "+CurrentBars[1]+", " + e.ToString());	}
		}
        protected override void OnBarUpdate()
        {

            try{
            if (CurrentBar <2) {Filter[0]=0;  Smoother[0]=(sma1[0]);return;}        
            
            Smoother[0]=(sma1[0]);            
            switch (Order)
            {  
                case 1: Filter[0]=sma2[0];                                                              break;
                case 2: Filter[0]=b*Filter[1]   - k1*Filter[2] + k2*Smoother[0];                        break;
                case 3: Filter[0]=(k3*Filter[1] - k4*Filter[2] + k5*Filter[3]     + k6*Smoother[0]);    break;					    
            }
			
            Values[0].Set(Filter[0]);
			if(Values[0].Count<2)return;
			if (Rising(Filter)) PlotColors[0][0] = upColor;
			else if (Falling(Filter)) PlotColors[0][0] = downColor;
			else PlotColors[0][0] = neutralColor;
            }catch (Exception e){	Print("Low Pass Cyle line 118, bar "+CurrentBars[0]+" : "+CurrentBars[1]+", " + e.ToString());	}
		
        }

        #region Properties
		
	   [Description("Color for Falling Plot")]
       [Category("Plot Line")]
        public Color ColorFalling
        {
            get { return downColor; }
            set { downColor = value; }
        }
		
		
		[Browsable(false)]
		public string ColorFallingSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(downColor); }
			set { downColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}

		
		[Description("Color for Rising Plot")]
       [Category("Plot Line")]
        public Color ColorRising
        {
            get { return upColor; }
            set { upColor = value; }
        }
		
		[Browsable(false)]
		public string ColorRisingSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(upColor); }
			set { upColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
				
		
		[Description("Color for Unchanging Plot")]
        [Category("Plot Line")]
        public Color ColorNeutral
        {
            get { return neutralColor; }
            set { neutralColor = value; }
        }
		
		[Browsable(false)]
		public string ColorNeutralSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(neutralColor); }
			set { neutralColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		} 
		
		
        [Browsable(false)]	// this line was put in by the stupid wizard and should be removed.
        [XmlIgnore()]		// this line ensures blah blah blah
		public DataSeries LPC
        {
            get { return Values[0]; }
        }

        [Description("Poles of Filtering, 1,2 or 3")]
        [GridCategory("Parameters")]
        public int Order
        {
            get { return order; }
            set { order = Math.Min(3,Math.Max(1, value)); }
        }

        [Description("")]
        [GridCategory("Parameters")]
        public int FilterPeriod
        {
            get { return filterPeriod; }
            set { filterPeriod = Math.Max(1, value); }
        }

        [Description("")]
        [GridCategory("Parameters")]
        public int PreSmooth
        {
            get { return preSmooth; }
            set { preSmooth = Math.Max(1, value); }
        }
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private LowPassFilter_v1[] cacheLowPassFilter_v1 = null;

        private static LowPassFilter_v1 checkLowPassFilter_v1 = new LowPassFilter_v1();

        /// <summary>
        /// Low Pass Filter Histogram
        /// </summary>
        /// <returns></returns>
        public LowPassFilter_v1 LowPassFilter_v1(int filterPeriod, int order, int preSmooth)
        {
            return LowPassFilter_v1(Input, filterPeriod, order, preSmooth);
        }

        /// <summary>
        /// Low Pass Filter Histogram
        /// </summary>
        /// <returns></returns>
        public LowPassFilter_v1 LowPassFilter_v1(Data.IDataSeries input, int filterPeriod, int order, int preSmooth)
        {
            if (cacheLowPassFilter_v1 != null)
                for (int idx = 0; idx < cacheLowPassFilter_v1.Length; idx++)
                    if (cacheLowPassFilter_v1[idx].FilterPeriod == filterPeriod && cacheLowPassFilter_v1[idx].Order == order && cacheLowPassFilter_v1[idx].PreSmooth == preSmooth && cacheLowPassFilter_v1[idx].EqualsInput(input))
                        return cacheLowPassFilter_v1[idx];

            lock (checkLowPassFilter_v1)
            {
                checkLowPassFilter_v1.FilterPeriod = filterPeriod;
                filterPeriod = checkLowPassFilter_v1.FilterPeriod;
                checkLowPassFilter_v1.Order = order;
                order = checkLowPassFilter_v1.Order;
                checkLowPassFilter_v1.PreSmooth = preSmooth;
                preSmooth = checkLowPassFilter_v1.PreSmooth;

                if (cacheLowPassFilter_v1 != null)
                    for (int idx = 0; idx < cacheLowPassFilter_v1.Length; idx++)
                        if (cacheLowPassFilter_v1[idx].FilterPeriod == filterPeriod && cacheLowPassFilter_v1[idx].Order == order && cacheLowPassFilter_v1[idx].PreSmooth == preSmooth && cacheLowPassFilter_v1[idx].EqualsInput(input))
                            return cacheLowPassFilter_v1[idx];

                LowPassFilter_v1 indicator = new LowPassFilter_v1();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.FilterPeriod = filterPeriod;
                indicator.Order = order;
                indicator.PreSmooth = preSmooth;
                Indicators.Add(indicator);
                indicator.SetUp();

                LowPassFilter_v1[] tmp = new LowPassFilter_v1[cacheLowPassFilter_v1 == null ? 1 : cacheLowPassFilter_v1.Length + 1];
                if (cacheLowPassFilter_v1 != null)
                    cacheLowPassFilter_v1.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheLowPassFilter_v1 = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Low Pass Filter Histogram
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.LowPassFilter_v1 LowPassFilter_v1(int filterPeriod, int order, int preSmooth)
        {
            return _indicator.LowPassFilter_v1(Input, filterPeriod, order, preSmooth);
        }

        /// <summary>
        /// Low Pass Filter Histogram
        /// </summary>
        /// <returns></returns>
        public Indicator.LowPassFilter_v1 LowPassFilter_v1(Data.IDataSeries input, int filterPeriod, int order, int preSmooth)
        {
            return _indicator.LowPassFilter_v1(input, filterPeriod, order, preSmooth);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Low Pass Filter Histogram
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.LowPassFilter_v1 LowPassFilter_v1(int filterPeriod, int order, int preSmooth)
        {
            return _indicator.LowPassFilter_v1(Input, filterPeriod, order, preSmooth);
        }

        /// <summary>
        /// Low Pass Filter Histogram
        /// </summary>
        /// <returns></returns>
        public Indicator.LowPassFilter_v1 LowPassFilter_v1(Data.IDataSeries input, int filterPeriod, int order, int preSmooth)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.LowPassFilter_v1(input, filterPeriod, order, preSmooth);
        }
    }
}
#endregion
